@@ -90,7 +90,7 @@
 /*
  * Version Information
  */
-#define DRIVER_VERSION "v2.4.30-pc0.7.9-8"
+#define DRIVER_VERSION "v2.4.30-pc0.8.2-1"
 #define DRIVER_AUTHOR "Vojtech Pavlik <vojtech@suse.cz>"
 #ifndef __JEJ_DEBUG
 #define DRIVER_DESC "USB Wacom Graphire and Wacom Intuos tablet driver (LINUXWACOM)"
@@ -142,7 +142,7 @@ static void wacom_pl_irq(struct urb *urb)
 	struct wacom *wacom = urb->context;
 	unsigned char *data = wacom->data;
 	struct input_dev *dev = &wacom->dev;
-	int prox, pressure, id;
+	int prox, pressure;
 
 	if (urb->status) return;
 
@@ -183,7 +183,7 @@ static void wacom_pl_irq(struct urb *urb)
 				/* report out proximity for previous tool */
 				input_report_abs(dev, ABS_MISC, wacom->id[0]); /* report tool id */
 				input_report_key(dev, wacom->tool[1], 0);
-				input_event(dev, EV_MSC, MSC_SERIAL, id);
+				input_event(dev, EV_MSC, MSC_SERIAL, wacom->id[0]);
 				wacom->tool[1] = BTN_TOOL_PEN;
 				return;
 			}
@@ -216,7 +216,7 @@ static void wacom_pl_irq(struct urb *urb)
 	wacom->tool[0] = prox; /* Save proximity state */
 	/* end of proximity code */
 	
-	input_event(dev, EV_MSC, MSC_SERIAL, id);
+	input_event(dev, EV_MSC, MSC_SERIAL, wacom->id[0]);
 }
 
 static void wacom_ptu_irq(struct urb *urb)
@@ -532,6 +532,7 @@ static int wacom_intuos_inout(struct urb *urb)
 			input_report_key(dev, BTN_STYLUS2, 0);
 			input_report_key(dev, BTN_TOUCH, 0);
 			input_report_abs(dev, ABS_WHEEL, 0);
+			input_report_abs(dev, ABS_Z, 0);
 		}
 		input_report_key(dev, wacom->tool[idx], 0);
 		input_report_abs(dev, ABS_MISC, 0); /* reset tool id */
@@ -593,7 +594,7 @@ static void wacom_intuos_irq(struct urb *urb)
 	/* tool index */
 	idx = data[1] & 0x01;
 
-	/* pad packets. Works as a second tool and is alway in prox */
+	/* pad packets. Works as a second tool */
 	if (data[0] == 12 && (strstr(wacom->features->name, "Intuos3") ||
 			strstr(wacom->features->name, "Cintiq")) )
 	{
@@ -615,11 +616,13 @@ static void wacom_intuos_irq(struct urb *urb)
 		input_report_abs(dev, ABS_RY, ((data[3] & 0x1f) << 8) | data[4]);
 
 		if((data[5] & 0x1f) | (data[6] & 0x1f) | (data[1] & 0x1f) | 
-			data[2] | (data[3] & 0x1f) | data[4])
+			data[2] | (data[3] & 0x1f) | data[4]) {
 			input_report_key(dev, wacom->tool[1], 1);
-		else
+			input_report_abs(dev, ABS_MISC, PAD_DEVICE_ID);
+		} else {
 			input_report_key(dev, wacom->tool[1], 0);
-		input_report_abs(dev, ABS_MISC, PAD_DEVICE_ID);
+			input_report_abs(dev, ABS_MISC, 0);
+		}
 		input_event(dev, EV_MSC, MSC_SERIAL, 0xffffffff);
 		return;
 	}
