@@ -80,16 +80,17 @@
  *    v1.30-j0.6.4 - new release
  *    v1.30-j0.6.5 - added Intuos3
  *    v1.30-j0.6.6 - new release
- *    v1.30-j0.6.7 - fixed a Graphire bug
+ *    v1.30-j0.6.7 - fixed a graphire bug
  *    v1.30-j0.6.8 - added Cintiq 21UX
  *    v1.30-j0.6.9 - fixed a Cintiq 21UX bug and cleaned Graphire code
- *    v1.30-j0.7.0 - new release
- *    v1.30-j0.7.1 - added DTF720, DTU 710, G4
+ *    v1.30-j0.7.0 - added Volito 2 and PenStation/PenPartner 2
+ *    v1.30-j0.7.1 - added DTU 710, DTF 720, Graphire4
  *    v1.30-j0.7.3 - added DTF 521, I3 12x12, and I3 12x19
- *    v1.30-j0.7.5 - Support tablet buttons/keys  
+ *    v1.30-j0.7.5 - Support tablet buttons/keys 
  *    v1.30-j0.7.7 - Support Intuos outbound tracking
  *    v1.30-j0.7.8 - Added Bamboo
  *    v1.30-j0.7.9 - added Bamboo1, Bamboo Fun, and Hummingbird
+ *    v1.30-j0.7.9-3 - Sync with wcmUSB.c
  *    v1.30-j0.7.9-8 - Added Cintiq 20WSX
  */
 
@@ -121,7 +122,7 @@
 /*
  * Version Information
  */
-#define DRIVER_VERSION "v1.30-j0.7.9-8"
+#define DRIVER_VERSION "v1.30-j0.8.2-1"
 #define DRIVER_AUTHOR "Vojtech Pavlik <vojtech@suse.cz>"
 #ifndef __JEJ_DEBUG
 #define DRIVER_DESC "USB Wacom Graphire and Wacom Intuos tablet driver (LINUXWACOM)"
@@ -170,7 +171,7 @@ struct wacom {
 	int open;
 	__u32 serial[2];
 	int id[2];
-	
+
 	struct list_head event_list;
 	struct semaphore kwacomd_sem;
 	unsigned int ifnum;
@@ -193,7 +194,7 @@ static void wacom_pl_irq(struct urb *urb)
 	struct wacom *wacom = urb->context;
 	unsigned char *data = wacom->data;
 	struct input_dev *dev = &wacom->dev;
-	int prox, pressure, id;
+	int prox, pressure;
 
 	if (urb->status) return;
 
@@ -233,8 +234,8 @@ static void wacom_pl_irq(struct urb *urb)
 			/* was entered with stylus2 pressed */
 			if (wacom->tool[1] == BTN_TOOL_RUBBER && !(data[4] & 0x20) ) {
 				/* report out proximity for previous tool */
-				input_report_abs(dev, ABS_MISC, wacom->id[0]); /* report tool id */
 				input_report_key(dev, wacom->tool[1], 0);
+				input_report_abs(dev, ABS_MISC, wacom->id[0]); /* report tool id */
 				input_event(dev, EV_MSC, MSC_SERIAL, wacom->id[0]);
 				wacom->tool[1] = BTN_TOOL_PEN;
 				return;
@@ -263,7 +264,7 @@ static void wacom_pl_irq(struct urb *urb)
 			wacom->tool[1] = BTN_TOOL_PEN;
 		}
 		input_report_key(dev, wacom->tool[1], prox);
-		input_report_abs(dev, ABS_MISC, wacom->id[0]);
+		input_report_abs(dev, ABS_MISC, wacom->id[0]); 
 	}
 
 	wacom->tool[0] = prox; /* Save proximity state */
@@ -302,14 +303,13 @@ static void wacom_ptu_irq(struct urb *urb)
 	input_report_abs(dev, ABS_MISC, wacom->id[0]); /* report tool id */
 	input_report_abs(dev, ABS_X, data[3] << 8 | data[2]);
 	input_report_abs(dev, ABS_Y, data[5] << 8 | data[4]);
-	input_report_abs(dev, ABS_PRESSURE, (data[6]|(data[7] & 0x01) << 8));
+	input_report_abs(dev, ABS_PRESSURE, (data[6]|data[7] << 8));
 	input_report_key(dev, BTN_STYLUS, data[1] & 0x02);
 	input_report_key(dev, BTN_STYLUS2, data[1] & 0x10);
 
 	input_event(dev, EV_MSC, MSC_SERIAL, wacom->id[0]);
 }
 
-
 static void wacom_penpartner_irq(struct urb *urb)
 {
 	struct wacom *wacom = urb->context;
@@ -355,7 +355,6 @@ static void wacom_penpartner_irq(struct urb *urb)
         }
 }
 
-
 static void wacom_graphire_irq(struct urb *urb)
 {
 	struct wacom *wacom = urb->context;
@@ -393,9 +392,8 @@ static void wacom_graphire_irq(struct urb *urb)
 			case 2: /* Mouse with wheel */
 				input_report_key(dev, BTN_MIDDLE, data[1] & 0x04);
 				if ( strstr(wacom->features->name, "Graphire4") || 
-						strstr(wacom->features->name, "Bamboo") )
-				{
-					rw = data[7] & 0x04 ? (data[7] & 0x03)-4 : (data[7] & 0x03);
+						strstr(wacom->features->name, "Bamboo") ) {
+					rw = data[7] & 0x04 ? (data[7] & 0x03)-4 : data[7] & 0x03;
 					input_report_rel(dev, REL_WHEEL, -rw);
 				} else
 					input_report_rel(dev, REL_WHEEL, -(signed char) data[6]);
@@ -440,10 +438,10 @@ static void wacom_graphire_irq(struct urb *urb)
 			input_report_key(dev, BTN_STYLUS2, 0);
 		}
 		wacom->id[0] = 0;
-		input_report_abs(dev, ABS_MISC, 0); /* reset tool id */	
-		input_report_key(dev, wacom->tool[0], 0);
+		input_report_abs(dev, ABS_MISC, 0); /* reset tool id */
+ 		input_report_key(dev, wacom->tool[0], 0);
 	}
-	input_event(dev, EV_MSC, MSC_SERIAL, id);
+	input_event(dev, EV_MSC, MSC_SERIAL, wacom->id[0]);
 
 	/* send pad data */
 	if ( strstr(wacom->features->name, "Graphire4") ) {
@@ -451,6 +449,7 @@ static void wacom_graphire_irq(struct urb *urb)
 			wacom->id[1] = PAD_DEVICE_ID;
 			input_report_key(dev, BTN_0, (data[7] & 0x40));
 			input_report_key(dev, BTN_4, (data[7] & 0x80));
+
 			rw = ((data[7] & 0x18) >> 3) - ((data[7] & 0x20) >> 3);
 			input_report_rel(dev, REL_WHEEL, rw);
 			input_report_key(dev, BTN_TOOL_FINGER, 0xf0);
@@ -458,8 +457,8 @@ static void wacom_graphire_irq(struct urb *urb)
 			input_event(dev, EV_MSC, MSC_SERIAL, 0xf0);
 		} else if (wacom->id[1]) {
 			wacom->id[1] = 0;
-			input_report_key(dev, BTN_0, (data[7] & 0x40));
-			input_report_key(dev, BTN_4, (data[7] & 0x80));
+                        input_report_key(dev, BTN_0, (data[7] & 0x40));
+                        input_report_key(dev, BTN_4, (data[7] & 0x80));
 			input_report_rel(dev, REL_WHEEL, 0);
 			input_report_key(dev, BTN_TOOL_FINGER, 0);
 			input_report_abs(dev, ABS_MISC, 0);
@@ -589,9 +588,10 @@ static int wacom_intuos_inout(struct urb *urb)
 			input_report_key(dev, BTN_STYLUS2, 0);
 			input_report_key(dev, BTN_TOUCH, 0);
 			input_report_abs(dev, ABS_WHEEL, 0);
+			input_report_abs(dev, ABS_Z, 0);
 		}
-		input_report_key(dev, wacom->tool[idx], 0);
 		input_report_abs(dev, ABS_MISC, 0); /* reset tool id */
+		input_report_key(dev, wacom->tool[idx], 0);
 		input_event(dev, EV_MSC, MSC_SERIAL, wacom->serial[idx]);
 		return 1;
 	}
@@ -651,9 +651,9 @@ static void wacom_intuos_irq(struct urb *urb)
 	/* tool index */
 	idx = data[1] & 0x01;
 
-	/* pad packets. Works as a second tool and is alway in prox */
-	if (data[0] == 12&& (strstr(wacom->features->name, "Intuos3") ||
-			strstr(wacom->features->name, "Cintiq")))
+	/* pad packets. Works as a second tool */
+	if (data[0] == 12 && (strstr(wacom->features->name, "Intuos3") ||
+			strstr(wacom->features->name, "Cintiq")) )
 	{
 		/* initiate the pad as a device */
 		if (wacom->tool[1] != BTN_TOOL_FINGER)
@@ -673,11 +673,13 @@ static void wacom_intuos_irq(struct urb *urb)
 		input_report_abs(dev, ABS_RY, ((data[3] & 0x1f) << 8) | data[4]);
 
 		if((data[5] & 0x1f) | (data[6] & 0x1f) | (data[1] & 0x1f) | 
-			data[2] | (data[3] & 0x1f) | data[4])
+			data[2] | (data[3] & 0x1f) | data[4]) {
 			input_report_key(dev, wacom->tool[1], 1);
-		else
+			input_report_abs(dev, ABS_MISC, PAD_DEVICE_ID);
+		} else {
 			input_report_key(dev, wacom->tool[1], 0);
-		input_report_abs(dev, ABS_MISC, PAD_DEVICE_ID);
+			input_report_abs(dev, ABS_MISC, 0);
+		}
 		input_event(dev, EV_MSC, MSC_SERIAL, 0xffffffff);
 		return;
 	}
@@ -760,10 +762,11 @@ static void wacom_intuos_irq(struct urb *urb)
 			/* I3 2D mouse side buttons */	
 			if (strstr(wacom->features->name, "Intuos3"))
 			{
-				input_report_key(dev, BTN_SIDE,   data[8] & 0x40);
-				input_report_key(dev, BTN_EXTRA,  data[8] & 0x20);
+				input_report_key(dev, BTN_SIDE,  data[8] & 0x40);
+				input_report_key(dev, BTN_EXTRA, data[8] & 0x20);
 			}
 		}
+
 		/* lens cursor packets */
 		else if ( !strstr(wacom->features->name, "Intuos3") )
 		{
@@ -775,7 +778,7 @@ static void wacom_intuos_irq(struct urb *urb)
 		}
 	}
 
-	input_report_abs(dev, ABS_MISC, wacom->id[idx]);
+	input_report_abs(dev, ABS_MISC, wacom->id[idx]); /* report tool id */
 	input_report_key(dev, wacom->tool[idx], 1);
 	input_event(dev, EV_MSC, MSC_SERIAL, wacom->serial[idx]);
 }
@@ -791,7 +794,7 @@ static void wacom_intuos_irq(struct urb *urb)
 #define WACOM_INTUOS_BUTTONS	(BIT(BTN_SIDE) | BIT(BTN_EXTRA))
 #define WACOM_INTUOS3S_BUTTONS	(WACOM_INTUOS_BUTTONS | BIT(BTN_0) | BIT(BTN_1) | BIT(BTN_2) | BIT(BTN_3))
 #define WACOM_INTUOS3_BUTTONS	(WACOM_INTUOS3S_BUTTONS | BIT(BTN_4) | BIT(BTN_5) | BIT(BTN_6) | BIT(BTN_7))
-#define WACOM_BEE_BUTTONS       (WACOM_INTUOS3_BUTTONS | BIT(BTN_8) | BIT(BTN_9))
+#define WACOM_BEE_BUTTONS	(WACOM_INTUOS3_BUTTONS | BIT(BTN_8) | BIT(BTN_9))
 #define WACOM_INTUOS_BITS	(BIT(EV_REL))
 #define WACOM_INTUOS_REL	(BIT(REL_WHEEL))
 #define WACOM_INTUOS_ABS	(BIT(ABS_TILT_X) | BIT(ABS_TILT_Y) | BIT(ABS_RZ) | BIT(ABS_THROTTLE))
@@ -1071,7 +1074,7 @@ static void *wacom_probe(struct usb_device *dev, unsigned int ifnum, const struc
 	wacom->dev.mscbit[0] |= BIT(MSC_SERIAL);
 
 	#ifdef __JEJ_DEBUG
-	printk(KERN_INFO __FILE__ ": Reporting max %d, %d\n",
+	printk(KERN_INFO "Reporting max %d, %d\n",
 		wacom->features->x_max, wacom->features->y_max);
 	#endif
 
