@@ -20,6 +20,8 @@
  * REVISION HISTORY
  *
  * 2007-05-25 0.1 - Initial release - span off from xf86Wacom.c
+ * 2008-05-14 0.2 - Rotate through routine xf86WcmRotateScreen
+ * 2008-06-26 0.3 - Added Capacity
  */
 
 
@@ -30,6 +32,7 @@
 
 extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
 extern void xf86WcmInitialScreens(LocalDevicePtr local);
+extern void xf86WcmRotateScreen(LocalDevicePtr local, int value);
 
 /*****************************************************************************
  * xf86WcmSetPadCoreMode
@@ -145,12 +148,8 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
-	WacomDevicePtr tmppriv;
 	char st[32];
-	int oldRotation;
-	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
 	WacomToolAreaPtr area = priv->toolarea;
-	WacomToolAreaPtr tmparea;
 
 	/* We don't reset options to the values that the driver are using.  
 	 * This eliminates confusion when driver is running on default values.
@@ -312,9 +311,13 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		xf86AddNewOption(local->options, "Speed", st);
 		break;
 	    case XWACOM_PARAM_ACCEL:
-		if ((value < 1) || (value > MAX_ACCEL)) return BadValue;
-		priv->accel = value-1;
-		xf86ReplaceIntOption(local->options, "Accel", priv->accel);
+		if ((value < 1) || (value > MAX_ACCEL)) 
+			return BadValue;
+		else if (priv->accel != value-1)
+		{
+			priv->accel = value-1;
+			xf86ReplaceIntOption(local->options, "Accel", priv->accel);
+		}
 		break;
 	    case XWACOM_PARAM_CLICKFORCE:
 		if ((value < 1) || (value > 21)) return BadValue;
@@ -355,6 +358,27 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 				xf86ReplaceStrOption(local->options, "TPCButton", "off");
 		}
 		break;
+	    case XWACOM_PARAM_TOUCH:
+		if ((value != 0) && (value != 1)) 
+			return BadValue;
+		else if (common->wcmTouch != value)
+		{
+			common->wcmTouch = value;
+			if (value)
+				xf86ReplaceStrOption(local->options, "Touch", "on");
+			else
+				xf86ReplaceStrOption(local->options, "Touch", "off");
+		}
+		break;
+	    case XWACOM_PARAM_CAPACITY:
+		if ((value < -1) || (value > 5)) 
+			return BadValue;
+		else if (common->wcmCapacity != value)
+		{
+			common->wcmCapacity = value;
+			xf86ReplaceIntOption(local->options, "Capacity", value);
+		}
+		break;
 	    case XWACOM_PARAM_CURSORPROX:
 		if (IsCursor (priv))
 		{
@@ -402,37 +426,69 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		else
 		{
 			int sNum = param - XWACOM_PARAM_TVRESOLUTION0;
-			int rX = 0, rY = 0;
-			if (sNum)
-			{
-				rX = priv->tvResolution[0];
-				rY = priv->tvResolution[1];
-				sNum++;
-			}
-			else
-			{
-				rX = priv->tvResolution[2];
-				rY = priv->tvResolution[3];
-			}
-			if ( priv->twinview == TV_ABOVE_BELOW )
-			{
-				rX = value & 0xffff;
-				rY += (value >> 16) & 0xffff;
-			}
-			else
-			{
-				rX += value & 0xffff;
-				rY = (value >> 16) & 0xffff;
-			}
+			int rX = value & 0xffff, rY = (value >> 16) & 0xffff;
 			if (rX > screenInfo.screens[0]->width ||
 					rY > screenInfo.screens[0]->height)
+			{
+				ErrorF("xf86WcmSetParam tvResolution out of range: " 
+					"ResX=%d ResY=%d \n", rX, rY);
 				return BadValue;
-			priv->tvResolution[sNum++] = value & 0xffff;
-			priv->tvResolution[sNum] = (value >> 16) & 0xffff;
-			xf86WcmChangeScreen(local, priv->screen_no);
+			}
+
 			DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam " 
-					"to ResX=%d ResY=%d \n",
-				value & 0xffff, (value >> 16) & 0xffff));
+				"tvResolutionX from %d to ResX=%d tvResolutionY "
+				"from %d toResY=%d \n",
+				priv->tvResolution[0], rX, priv->tvResolution[1], rY));
+
+			if ( priv->twinview == TV_ABOVE_BELOW )
+			{
+				if (sNum)
+				{
+					priv->tvResolution[1] = screenInfo.screens[0]->height - rY;
+					priv->tvResolution[2] = rX;
+					priv->tvResolution[3] = rY;
+				}
+				else
+				{
+					priv->tvResolution[0] = rX;
+					priv->tvResolution[1] = rY;
+					priv->tvResolution[3] = screenInfo.screens[0]->height - rY;
+				}
+			}
+			else
+			{
+				if (sNum)
+				{
+					priv->tvResolution[0] = screenInfo.screens[0]->width - rX;
+					priv->tvResolution[2] = rX;
+					priv->tvResolution[3] = rY;
+				}
+				else
+				{
+					priv->tvResolution[0] = rX;
+					priv->tvResolution[1] = rY;
+					priv->tvResolution[2] = screenInfo.screens[0]->width - rX;
+				}
+			}
+		}
+		/* initial screen info */
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0];
+		priv->screenBottomY[0] = priv->tvResolution[1];
+		if (priv->twinview == TV_ABOVE_BELOW)
+		{
+			priv->screenTopX[1] = 0;
+			priv->screenTopY[1] = priv->tvResolution[1];
+			priv->screenBottomX[1] = priv->tvResolution[2];
+			priv->screenBottomY[1] = priv->tvResolution[1] + priv->tvResolution[3];
+		}
+		if (priv->twinview == TV_LEFT_RIGHT)
+		{
+			priv->screenTopX[1] = priv->tvResolution[0];
+			priv->screenTopY[1] = 0;
+			priv->screenBottomX[1] = priv->tvResolution[0] + priv->tvResolution[2];
+			priv->screenBottomY[1] = priv->tvResolution[3];
 		}
 		break;
 	    }
@@ -452,106 +508,10 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		break;
 	   case XWACOM_PARAM_ROTATE:
 		if ((value < 0) || (value > 3)) return BadValue;
-		switch(value) {
-		  case ROTATE_NONE:
-			xf86ReplaceStrOption(local->options, "Rotate", "NONE");
-			break;
-		  case ROTATE_CW:
-			xf86ReplaceStrOption(local->options, "Rotate", "CW");
-			break;
-		  case ROTATE_CCW:
-			xf86ReplaceStrOption(local->options, "Rotate", "CCW");
-			break;
-		  case ROTATE_HALF:
-			xf86ReplaceStrOption(local->options, "Rotate", "HALF");
-			break;
-		  default:
-			return BadValue;
-		}
-		oldRotation = common->wcmRotate;
-		oldMaxX = priv->wcmMaxX;
-		oldMaxY = priv->wcmMaxY;
-		common->wcmRotate = value;
-		if (((oldRotation == ROTATE_NONE || oldRotation == ROTATE_HALF) && 
-			(value == ROTATE_CW || value == ROTATE_CCW)) ||
-		     ((oldRotation == ROTATE_CW || oldRotation == ROTATE_CCW) 
-			&& (value == ROTATE_NONE || value == ROTATE_HALF)))
-		{
-		    priv->wcmMaxX = oldMaxY;
-		    priv->wcmMaxY = oldMaxX;
-		}
-
-		/* rotate all devices at once! else they get misaligned */
-		for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
-		{
-		    tmparea = tmppriv->toolarea;
-		    /* recover the unrotated xy-rectangles */
-		    switch (oldRotation) {
-		      case ROTATE_CW:
-			tmpTopX = oldMaxY - tmppriv->bottomY;
-			tmpBottomX = oldMaxY - tmppriv->topY;
-			tmpTopY = tmppriv->topX;
-			tmpBottomY = tmppriv->bottomX;
-			break;
-		      case ROTATE_CCW:
-			tmpTopX = tmppriv->topY;
-			tmpBottomX = tmppriv->bottomY;
-			tmpTopY = oldMaxX - tmppriv->bottomX;
-			tmpBottomY = oldMaxX - tmppriv->topX;
-			break;
-		      case ROTATE_HALF:
-			tmpTopX = oldMaxX - tmppriv->bottomX;
-			tmpBottomX = oldMaxX - tmppriv->topX;
-			tmpTopY = oldMaxY - tmppriv->bottomY;
-			tmpBottomY = oldMaxY - tmppriv->topY;
-			break;
-		      default: /* ROTATE_NONE */
-			tmpTopX = tmppriv->topX;
-			tmpBottomX = tmppriv->bottomX;
-			tmpTopY = tmppriv->topY;
-			tmpBottomY = tmppriv->bottomY;
-			break;
-		    } 
-		    /* and rotate them back */
-		    switch (value) {
-		      case ROTATE_CW:
-			tmparea->topX = tmppriv->topX = tmpTopY;
-			tmparea->bottomX = tmppriv->bottomX = tmpBottomY;
-			tmparea->topY = tmppriv->topY = priv->wcmMaxY - tmpBottomX;
-			tmparea->bottomY = tmppriv->bottomY = priv->wcmMaxY - tmpTopX;
-			break;
-		      case ROTATE_CCW:
-			tmparea->topX = tmppriv->topX = priv->wcmMaxX - tmpBottomY;
-			tmparea->bottomX = tmppriv->bottomX = priv->wcmMaxX - tmpTopY;
-			tmparea->topY = tmppriv->topY = tmpTopX;
-			tmparea->bottomY = tmppriv->bottomY = tmpBottomX;
-			break;
-		      case ROTATE_HALF:
-			tmparea->topX = tmppriv->topX = priv->wcmMaxX - tmpBottomX;
-			tmparea->bottomX = tmppriv->bottomX = priv->wcmMaxX - tmpTopX;
-			tmparea->topY = tmppriv->topY= priv->wcmMaxY - tmpBottomY;
-			tmparea->bottomY = tmppriv->bottomY = priv->wcmMaxY - tmpTopY;
-			break;
-		      default: /* ROTATE_NONE */
-			tmparea->topX = tmppriv->topX = tmpTopX;
-			tmparea->bottomX = tmppriv->bottomX = tmpBottomX;
-			tmparea->topY = tmppriv->topY = tmpTopY;
-			tmparea->bottomY = tmppriv->bottomY = tmpBottomY;
-			break;
-		    }
-
-		    xf86WcmInitialScreens(tmppriv->local);
-		    xf86WcmMappingFactor(tmppriv->local);
-		    xf86WcmInitialCoordinates(tmppriv->local, 0);
-		    xf86WcmInitialCoordinates(tmppriv->local, 1);
-
-		    xf86ReplaceIntOption(tmppriv->local->options, "TopX", tmppriv->topX);
-		    xf86ReplaceIntOption(tmppriv->local->options, "TopY", tmppriv->topY);
-		    xf86ReplaceIntOption(tmppriv->local->options, "BottomX", tmppriv->bottomX);
-		    xf86ReplaceIntOption(tmppriv->local->options, "BottomY", tmppriv->bottomY);
-		}
+		if (common->wcmRotate != value)
+			xf86WcmRotateScreen(local, value);
 		break;
-	      default:
+	   default:
 		DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam invalid param %d\n",param));
 		return BadMatch;
 	}
@@ -860,6 +820,10 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 		return priv->wcmMMonitor;
 	    case XWACOM_PARAM_TPCBUTTON:
 		return common->wcmTPCButton;
+	    case XWACOM_PARAM_TOUCH:
+		return common->wcmTouch;
+	    case XWACOM_PARAM_CAPACITY:
+		return common->wcmCapacity;
 	    case XWACOM_PARAM_CURSORPROX:
 		if (IsCursor (priv))
 			return common->wcmCursorProxoutDist;
@@ -887,6 +851,12 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 		return priv->screen_no;
 	    case XWACOM_PARAM_NUMSCREEN:
 		return priv->numScreen;
+	    case XWACOM_PARAM_XSCALING:
+#ifdef WCM_XORG_TABLET_SCALING
+		return 1;
+#else
+		return 0;
+#endif
 	}
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmGetParam invalid param %d\n", param));
 	return -1;
@@ -1025,6 +995,10 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 		return 1;
 	case XWACOM_PARAM_TPCBUTTON:
 		return common->wcmTPCButtonDefault;
+	case XWACOM_PARAM_TOUCH:
+		return common->wcmTouchDefault;
+	case XWACOM_PARAM_CAPACITY:
+		return common->wcmCapacityDefault;
 	case XWACOM_PARAM_PRESSCURVE:
 		if (!IsCursor (priv) && !IsPad (priv) && !IsTouch (priv))
 			return (0 << 24) | (0 << 16) | (100 << 8) | 100;
